/* DO NOT EDIT THIS FILE!  It is auto-generated from the option list in ./rsync.1.md! */

  rprintf(F,"--daemon                 run as an rsync daemon\n");
  rprintf(F,"--address=ADDRESS        bind to the specified address\n");
  rprintf(F,"--bwlimit=RATE           limit socket I/O bandwidth\n");
  rprintf(F,"--config=FILE            specify alternate rsyncd.conf file\n");
  rprintf(F,"--dparam=OVERRIDE, -M    override global daemon config parameter\n");
  rprintf(F,"--no-detach              do not detach from the parent\n");
  rprintf(F,"--port=PORT              listen on alternate port number\n");
  rprintf(F,"--log-file=FILE          override the \"log file\" setting\n");
  rprintf(F,"--log-file-format=FMT    override the \"log format\" setting\n");
  rprintf(F,"--sockopts=OPTIONS       specify custom TCP options\n");
  rprintf(F,"--verbose, -v            increase verbosity\n");
  rprintf(F,"--ipv4, -4               prefer IPv4\n");
  rprintf(F,"--ipv6, -6               prefer IPv6\n");
  rprintf(F,"--help, -h               show this help (when used with --daemon)\n");
